@@ -1,4 +1,4 @@
-/*  P A S - P. WOLFERS SOFTWARE: V2.1-E OF 31-AUG-2009 
+/*  P A S - P. WOLFERS SOFTWARE: V3.0-A OF 31-OCT-2014 
   ***  PASCAL II implementation in C. *** */
 
 /* Set specific Macro setting. */
@@ -52,24 +52,23 @@ static Blk( Rd, 120 ) = {
 };
 
 /* Define the local static Initialized section. */
-static Blk( Ri, 264 ) = {
+static Blk( Ri, 256 ) = {
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0, 255, 255, 255, 255,   0,   0,   0,   1,
+  0,   0,   0,   0, 255, 255, 255, 255,   0,   0,   0,   1,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   2,   0,  32, 144,  32,  32,  32,  32,   0,   0,   0,   0,
+  2,   0,  32, 144,  32,  32,  32,  32,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,
-  0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   4,   0,   0, 145,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   1,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   1,   0,   0,   1,   0,
+  4,   0,   0, 145,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0, 146,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-  0,   0,   0,   2,   0,   0,   0,   1,   0,   0,   1,   0,   0,   0,   0,   0,
+  0,   0,   0,   1,   0,   0,   0,   1,   0,   0,   1,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 146,
-  2,   0,   0,   0,   0,   0,   0,   0
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,
+  0,   0,   0,   1,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 146,   2,   0,   0,   0
 };
 
 /* Define the local static section. */
@@ -172,6 +171,25 @@ void PAS__ins_proc(V *F_pp, void F_pr(V F_fp));
 void PAS__exe_proc(V F_pp, V F_fp);
 
 
+/* Procedure/Function : " pas_$new "  */
+V PAS__NEW(SL F_size){
+  register V Ret;
+
+  /* Code of procedure/function */
+  Ret=PAS__MEM_ALLOC(F_size);
+  return(Ret);
+}
+
+
+/* Procedure/Function : " pas_$dispose "  */
+void PAS__DISPOSE(V *F_ptr){
+
+  /* Code of procedure/function */
+  PAS__MEM_FREE((*F_ptr));
+  (*F_ptr)=NULL;
+}
+
+
 /* Procedure/Function : " alloc_new_heap "  */
 void PAS__STD_PRC_00000000(SL F_sz){
   /* Local variable stored in C variable */
@@ -179,23 +197,22 @@ void PAS__STD_PRC_00000000(SL F_sz){
   V Rv1;
 
   /* Code of procedure/function */
-  F_sz=F_sz+2;
   if (PAS__heap_incr<128) PAS__heap_incr=128;
+  F_sz=F_sz+12;
   F_sz=(((F_sz+PAS__heap_incr)-1)/PAS__heap_incr)*PAS__heap_incr;
-  Ri.ub[0]=0;
+  F_sz=(F_sz+7)/8;
   Rv0=F_sz;
   Rv1=PAS__NEW((Rv0+1)*8+16);
   ((SL *)Rv1)[0]=Rv0;
-  Ri.ub[0]=1;
   ((SL *)Rv1)[1]=0;
-  ((V *)Rv1)[2]=Ri.v[1];
-  ((V *)Rv1)[3]=Ri.v[2];
-  Ri.v[1]=Rv1;
+  ((V *)Rv1)[2]=Ri.v[0];
+  ((V *)Rv1)[3]=Ri.v[1];
+  Ri.v[0]=Rv1;
 }
 
 
 /* Procedure/Function : " heap_alloc "  */
-V PAS__STD_heap_alloc(SL F_size){
+V PAS__HEAP_ALLOC(SL F_size){
   /* Local variable stored in C variable */
   register SL Rv0;
   register V Rv1;
@@ -206,56 +223,32 @@ V PAS__STD_heap_alloc(SL F_size){
   register V Ret;
 
   /* Code of procedure/function */
-  if (Ri.v[1]==NULL) {
-    Ri.ub[0]=0;
+  if (Ri.v[0]==NULL) {
     if (PAS__heap_isiz<128) PAS__heap_isiz=128;
     Rv0=PAS__heap_isiz;
-    Ri.v[1]=PAS__NEW((Rv0+1)*8+16);
-    ((SL *)Ri.v[1])[0]=Rv0;
-    Rv1=Ri.v[1];
+    Ri.v[0]=PAS__NEW((Rv0+1)*8+16);
+    ((SL *)Ri.v[0])[0]=Rv0;
+    Rv1=Ri.v[0];
     ((SL *)Rv1)[1]=0;
     ((V *)Rv1)[2]=NULL;
     ((V *)Rv1)[3]=NULL;
-    Ri.v[2]=NULL;
-    Ri.ub[0]=1;
+    Ri.v[1]=NULL;
   }
-  Rv2=((SL *)Ri.v[1])[1];
+  Rv2=((SL *)Ri.v[0])[1];
   Rv3=(F_size+7)/8;
   Rv4=Rv2+Rv3;
-  if (Rv4>=((SL *)Ri.v[1])[0]) {
+  if (Rv4>=((SL *)Ri.v[0])[0]) {
     PAS__STD_PRC_00000000(Rv3);
     Rv4=Rv3;
     Rv2=0;
   }
-  Rv5=Ri.v[1];
+  Rv5=Ri.v[0];
   ((SL *)Rv5)[1]=Rv4;
   Ret=cc_wild_pointer((((G *)Rv5)+Rv2+2));
   return(Ret);
 }
 
 
-/* Procedure/Function : " pas_$new "  */
-V PAS__NEW(SL F_size){
-  register V Ret;
-
-  /* Code of procedure/function */
-  if (Ri.ub[0]&&Ri.v[1]!=NULL) Ret=PAS__STD_heap_alloc(F_size);
-  else Ret=PAS__MEM_ALLOC(F_size);
-  return(Ret);
-}
-
-
-/* Procedure/Function : " pas_$dispose "  */
-void PAS__DISPOSE(V *F_ptr){
-
-  /* Code of procedure/function */
-  if (!Ri.ub[0]) {
-    PAS__MEM_FREE((*F_ptr));
-    (*F_ptr)=NULL;
-  }
-}
-
-
 /* Procedure/Function : " pas_$mark "  */
 void PAS__MARK(){
   /* Local variable stored in C variable */
@@ -265,22 +258,21 @@ void PAS__MARK(){
   register V Rv3;
 
   /* Code of procedure/function */
-  Ri.ub[0]=1;
-  if (Ri.v[1]!=NULL) {
-    Rv0=Ri.v[1];
-    Rv1=((SL *)Ri.v[1])[1];
+  if (Ri.v[0]!=NULL) {
+    Rv0=Ri.v[0];
+    Rv1=((SL *)Ri.v[0])[1];
   }
   else {
     Rv0=NULL;
     Rv1=0;
   }
-  Rv2=PAS__NEW(12);
+  Rv2=PAS__HEAP_ALLOC(12);
   Rv3=Rv2;
-  ((V *)Rv3)[0]=Ri.v[2];
-  if (Rv0==NULL) ((V *)Rv3)[1]=Ri.v[1];
+  ((V *)Rv3)[0]=Ri.v[1];
+  if (Rv0==NULL) ((V *)Rv3)[1]=Ri.v[0];
   else((V *)Rv3)[1]=Rv0;
   ((SL *)Rv3)[2]=Rv1;
-  Ri.v[2]=Rv2;
+  Ri.v[1]=Rv2;
 }
 
 
@@ -294,37 +286,22 @@ void PAS__RELEASE(){
   V Rv4;
 
   /* Code of procedure/function */
-  if ((Ri.ub[0]&&Ri.v[1]!=NULL)&&Ri.v[2]!=NULL) {
-    Rv0=Ri.v[2];
+  if (Ri.v[0]!=NULL&&Ri.v[1]!=NULL) {
+    Rv0=Ri.v[1];
     Rv1=((V *)Rv0)[0];
     Rv2=((V *)Rv0)[1];
     Rv3=((SL *)Rv0)[2];
-    if (Rv2!=NULL&&Rv2!=Ri.v[1]) {
-      Ri.ub[0]=0;
-      while (Rv2!=Ri.v[1]){
-        Rv4=((V *)Ri.v[1])[2];
-        PAS__DISPOSE(Ri.v+1);
-        Ri.v[1]=Rv4;
-      }
-      Ri.ub[0]=1;
+    if (Rv2!=NULL&&Rv2!=Ri.v[0]) while (Rv2!=Ri.v[0]){
+      Rv4=((V *)Ri.v[0])[2];
+      PAS__DISPOSE(Ri.v);
+      Ri.v[0]=Rv4;
     }
-    ((SL *)Ri.v[1])[1]=Rv3;
-    Ri.v[2]=Rv1;
+    ((SL *)Ri.v[0])[1]=Rv3;
+    Ri.v[1]=Rv1;
   }
 }
 
 
-/* Procedure/Function : " pas__heap_mode "  */
-UB PAS__HEAP_MODE(UB F_flg){
-  register UB Ret;
-
-  /* Code of procedure/function */
-  Ret=Ri.ub[0];
-  Ri.ub[0]=F_flg;
-  return(Ret);
-}
-
-
 /* Procedure/Function : " pas__get_env "  */
 SL PAS__GET_ENV_ARRAY(V *F_ptr, V F_src){
   /* Local variable stored in C variable */
@@ -473,15 +450,15 @@ void PAS__STD_vread_end(V F_fp){
 
 
   /* Code of procedure/function */
-  Ra.v[0]=Ri.v[7];
-  if (Ri.sl[13]>0) {
-    if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=(Ri.sl[12]-Ri.sl[13])+1;
+  Ra.v[0]=Ri.v[6];
+  if (Ri.sl[12]>0) {
+    if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=(Ri.sl[11]-Ri.sl[12])+1;
   }
   else {
-    Ri.sl[13]=0;
-    Ri.v[3]=Ri.s+72;
-    Ri.ul[17]=Ri.ul[17]|184549376;
-    if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=-1;
+    Ri.sl[12]=0;
+    Ri.v[2]=Ri.s+68;
+    Ri.ul[16]=Ri.ul[16]|184549376;
+    if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=-1;
   }
 }
 
@@ -494,15 +471,15 @@ void PAS__VREAD_INIT(V F_str, SL F_sz, SL *F_ip){
   /* Code of procedure/function */
   if (((V) F_ip)!=NULL) Rv0=(*F_ip);
   else Rv0=1;
-  Ri.v[3]=(((C *)F_str)+Rv0-1);
-  Ri.v[4]=F_str;
-  Ri.v[7]=((V) F_ip);
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_vread_end);
-  Ri.sl[12]=F_sz;
-  Ri.sl[13]=(F_sz-Rv0)+1;
-  Ri.ul[17]=8337;
-  PAS__curr_iptr=Ri.s+12;
+  Ri.v[2]=(((C *)F_str)+Rv0-1);
+  Ri.v[3]=F_str;
+  Ri.v[6]=((V) F_ip);
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_vread_end);
+  Ri.sl[11]=F_sz;
+  Ri.sl[12]=(F_sz-Rv0)+1;
+  Ri.ul[16]=8337;
+  PAS__curr_iptr=Ri.s+8;
 }
 
 
@@ -516,17 +493,17 @@ void PAS__STD_twrite_end(V F_fp){
   register int Rf0;
 
   /* Code of procedure/function */
-  Ra.v[0]=Ri.v[7];
-  if (Ri.v[7]!=NULL) ((SL *)Ra.v[0])[0]=Ri.sl[13];
+  Ra.v[0]=Ri.v[6];
+  if (Ri.v[6]!=NULL) ((SL *)Ra.v[0])[0]=Ri.sl[12];
   else {
-    Rv0=Ri.sl[13]+1;
-    for( Rf0=(Ri.sl[12]+1)-(Ri.sl[13]+1);Rf0>0;Rf0-- ) {
-      ((C *)Ri.v[4])[Rv0]=32;
+    Rv0=Ri.sl[12]+1;
+    for( Rf0=(Ri.sl[11]+1)-(Ri.sl[12]+1);Rf0>0;Rf0-- ) {
+      ((C *)Ri.v[3])[Rv0]=32;
       Rv0++;
     }
   }
-  if (33554432&Ri.ul[17]) PAS__ERROR(54);
-  else Ri.ul[17]=Ri.ul[17]|167772160;
+  if (33554432&Ri.ul[16]) PAS__ERROR(54);
+  else Ri.ul[16]=Ri.ul[16]|167772160;
 }
 
 
@@ -538,15 +515,15 @@ void PAS__TWRITE_INIT(V F_str, SL F_sz, SL *F_ip){
   /* Code of procedure/function */
   if (((V) F_ip)!=NULL) Rv0=(*F_ip);
   else Rv0=0;
-  Ri.v[3]=(((C *)F_str)+(Rv0+1)-1);
-  Ri.v[4]=F_str;
-  Ri.v[7]=((V) F_ip);
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_twrite_end);
-  Ri.sl[12]=F_sz;
-  Ri.sl[13]=Rv0;
-  Ri.ul[17]=8338;
-  PAS__curr_optr=Ri.s+12;
+  Ri.v[2]=(((C *)F_str)+(Rv0+1)-1);
+  Ri.v[3]=F_str;
+  Ri.v[6]=((V) F_ip);
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_twrite_end);
+  Ri.sl[11]=F_sz;
+  Ri.sl[12]=Rv0;
+  Ri.ul[16]=8338;
+  PAS__curr_optr=Ri.s+8;
 }
 
 
@@ -554,9 +531,9 @@ void PAS__TWRITE_INIT(V F_str, SL F_sz, SL *F_ip){
 void PAS__STD_vwrite_end(V F_fp){
 
   /* Code of procedure/function */
-  ((UB *)Ri.v[9])[1]=Ri.sl[13];
-  if (33554432&Ri.ul[17]) PAS__ERROR(54);
-  else Ri.ul[17]=Ri.ul[17]|167772160;
+  ((UB *)Ri.v[8])[1]=Ri.sl[12];
+  if (33554432&Ri.ul[16]) PAS__ERROR(54);
+  else Ri.ul[16]=Ri.ul[16]|167772160;
 }
 
 
@@ -564,20 +541,20 @@ void PAS__STD_vwrite_end(V F_fp){
 void PAS__VWRITE_INIT(V F_str, UB F_bini){
 
   /* Code of procedure/function */
-  Ri.v[4]=F_str+2;
-  Ri.v[9]=F_str;
-  PAS__ins_proc(Ri.v+8,PAS__STD_vwrite_end);
-  Ri.sl[12]=((UB *)F_str)[0];
+  Ri.v[3]=F_str+2;
+  Ri.v[8]=F_str;
+  PAS__ins_proc(Ri.v+7,PAS__STD_vwrite_end);
+  Ri.sl[11]=((UB *)F_str)[0];
   if (F_bini) {
-    Ri.v[3]=F_str+2;
-    Ri.sl[13]=0;
+    Ri.v[2]=F_str+2;
+    Ri.sl[12]=0;
   }
   else {
-    Ri.v[3]=(((C *)F_str)+(((UB *)F_str)[1]+1)+1);
-    Ri.sl[13]=((UB *)F_str)[1];
+    Ri.v[2]=(((C *)F_str)+(((UB *)F_str)[1]+1)+1);
+    Ri.sl[12]=((UB *)F_str)[1];
   }
-  Ri.ul[17]=8338;
-  PAS__curr_optr=Ri.s+12;
+  Ri.ul[16]=8338;
+  PAS__curr_optr=Ri.s+8;
 }
 
 
@@ -839,7 +816,7 @@ void PAS__STD_get_file_elem(V F_fp){
           }
           else {
             ((SL *)Rv0)[10]=0;
-            if (Rv1==0) ((V *)Rv0)[0]=Ri.s+72;
+            if (Rv1==0) ((V *)Rv0)[0]=Ri.s+68;
             ((UL *)Rv0)[14]=(((UL *)Rv0)[14]|67108864)&~16777216;
           }
         }
@@ -1135,7 +1112,7 @@ void PAS__WRITE_EOLN(){
   /* Code of procedure/function */
   if (PAS__curr_optr!=NULL) {
     Rv0=PAS__curr_optr;
-    if (32&((UL *)Rv0)[14]||PAS__curr_optr==Ri.s+12) {
+    if (32&((UL *)Rv0)[14]||PAS__curr_optr==Ri.s+8) {
       if (8192&((UL *)Rv0)[14]) PAS__exe_proc(((V *)Rv0)[5],PAS__curr_optr);
       else PAS_put_buffer(PAS__curr_optr);
     }
@@ -1913,36 +1890,36 @@ void PAS__INIT(SL F_parc, V F_parv, V F_penv){
   /* Code of procedure/function */
   Ra.ub[0]=255;
   CC_TRAP_INIT();
-  PAS__f_input=Ri.s+76;
-  PAS__f_output=Ri.s+136;
-  PAS__f_error=Ri.s+196;
+  PAS__f_input=Ri.s+72;
+  PAS__f_output=Ri.s+132;
+  PAS__f_error=Ri.s+192;
   if (PAS__GETENV(Ra.s,Rd.s+47)<=0) PAS__SETENV(Rd.s+47,Rd.s+51,1);
   if (PAS__GETENV(Ra.s,Rd.s+61)<=0) PAS__SETENV(Rd.s+61,Rd.s+65,1);
-  Ri.sl[26]=0;
-  Ri.sl[27]=1;
-  Ri.sl[28]=256;
-  Ri.ul[33]=67109009;
-  if (isatty(Ri.sl[26])!=0) PAS__STD_PRC_00000001(Ri.sl[26],Ra.s);
+  Ri.sl[25]=0;
+  Ri.sl[26]=1;
+  Ri.sl[27]=256;
+  Ri.ul[32]=67109009;
+  if (isatty(Ri.sl[25])!=0) PAS__STD_PRC_00000001(Ri.sl[25],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+76);
-  PAS__STD_init_a_file(Ri.s+76,Ra.s);
+  PAS__STD_init_a_file(Ri.s+72,Ra.s);
+  Ri.sl[40]=1;
   Ri.sl[41]=1;
-  Ri.sl[42]=1;
-  Ri.sl[43]=256;
-  Ri.ul[48]=146;
-  if (isatty(Ri.sl[41])!=0) PAS__STD_PRC_00000001(Ri.sl[41],Ra.s);
+  Ri.sl[42]=256;
+  Ri.ul[47]=146;
+  if (isatty(Ri.sl[40])!=0) PAS__STD_PRC_00000001(Ri.sl[40],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+87);
-  PAS__STD_init_a_file(Ri.s+136,Ra.s);
-  Ri.sl[56]=2;
-  Ri.sl[57]=1;
-  Ri.sl[58]=256;
-  Ri.ul[63]=146;
-  if (isatty(Ri.sl[56])!=0) PAS__STD_PRC_00000001(Ri.sl[56],Ra.s);
+  PAS__STD_init_a_file(Ri.s+132,Ra.s);
+  Ri.sl[55]=2;
+  Ri.sl[56]=1;
+  Ri.sl[57]=256;
+  Ri.ul[62]=146;
+  if (isatty(Ri.sl[55])!=0) PAS__STD_PRC_00000001(Ri.sl[55],Ra.s);
   else PAS__STR_TO_STR(Ra.s,Rd.s+99);
-  PAS__STD_init_a_file(Ri.s+196,Ra.s);
-  Ri.sl[10]=-1;
-  Ri.sl[11]=1;
-  Ri.sl[12]=0;
-  Ri.ul[17]=33562768;
+  PAS__STD_init_a_file(Ri.s+192,Ra.s);
+  Ri.sl[9]=-1;
+  Ri.sl[10]=1;
+  Ri.sl[11]=0;
+  Ri.ul[16]=33562768;
   PAS__argc=F_parc;
   Rv0=0;
   for( Rf1=(F_parc-1)+1;Rf1>0;Rf1-- ) {
@@ -1961,7 +1938,7 @@ void PAS__INIT(SL F_parc, V F_parv, V F_penv){
         Rv4++;
       }
     }
-    else PAS__argv.v[Rv0]=Ri.s+256;
+    else PAS__argv.v[Rv0]=Ri.s+252;
     Rv0++;
   }
   Rv5=F_parc;
